function [x,y,typ] = hart_filtfilt(job,arg1,arg2)
//Zero-phase digital filtering
//
// Block Screenshot
//
// Description
//
//This block filters vectors. The vector can either filtered normally though a iir filter or using a zero-phase filtering.
//
// Dialog box
//
// filt (0) or filtfilt (1):           0 - normal iir filter. Can be used for vectors. 
//1 - the data is filtered twice: one time forward and one time backward. This means that the filtered data has no time delay caused by filterin
// No of Samples:  Number of input and output vector dimension
// order:  filter order
// num:  Numerator of the iir filter 
// den: Denominator of the iir filter 
//
// Default Properties
//
// always active: yes
// direct-feedthrough: no
// zero-crossing: no
// mode: no
// regular inputs:  port 1 : size [1,1] / type 1
// regular outputs: port 1 : size [1,1] / type 1
// number/sizes of activation inputs: 1
// number/sizes of activation outputs: 0
// continuous-time state: no
// discrete-time state: yes
// object discrete-time state: no
// name of computational function: rt_filtfilt
//  
//
// Interfacing Function
// {hart_toolbox}/modules/hart_base/macros/hart_filtfilt.sci
// Computational Function
// {hart_toolbox}/modules/hart_base/src/c/hart_filtfiltr.c
// Authors
// Holger Nahrstaedt
//

  x=[];y=[];typ=[];
  select job
  case 'plot' then
    exprs=arg1.graphics.exprs;
     samples=exprs(2);filtfilt=exprs(1);
    standard_draw(arg1)
  case 'getinputs' then
    [x,y,typ]=standard_inputs(arg1)
  case 'getoutputs' then
    [x,y,typ]=standard_outputs(arg1)
  case 'getorigin' then
    [x,y]=standard_origin(arg1)
  case 'set' then
    x=arg1
    model=arg1.model;graphics=arg1.graphics;
    exprs=graphics.exprs;
    while %t do
      try
       getversion('scilab');
      [ok,filtfilt,samples,order,num,den,exprs]=..
      scicos_getvalue('Set filtfilt block parameters',..
      [//'Channels:';
       'filt (0) or filtfilt (1):';
       'No of Samples:';
	'order: ';
	'num: ';
	'den: ';],..
      list('vec',-1,'vec',-1,'vec',-1,'pol',1,'pol',1),exprs)
      catch
      [ok,filtfilt,samples,order,num,den,exprs]=..
      getvalue('Set filtfilt block parameters',..
      [//'Channels:';
       'filt (0) or filtfilt (1):';
       'No of Samples:';
	'order: ';
	'num: ';
	'den: ';],..
      list('vec',-1,'vec',-1,'vec',-1,'pol',1,'pol',1),exprs)
      end
      if ~ok then break,end
      ch=1;
      outport=ch;
      inport=ch;
       if exists('inport') then in=ones(inport,1)*samples, else in=1, end
      if exists('outport') then out=ones(outport,1)*samples, else out=1, end
      [model,graphics,ok]=check_io(model,graphics,in,out,[],[])
      if ok then
        graphics.exprs=exprs;
        model.ipar=[ch;
		    samples;
		    order;
		    filtfilt];
	cnum=coeff(num);
	cden=coeff(den);
        model.rpar=[cnum(:);cden(:);];
        model.dstate=[];
        x.graphics=graphics;x.model=model
        break
      end
    end
  case 'define' then
    filtfilt=1;
    ch=1;
    samples=50;
    order=1;
    num='1';
    den='%z-1';
    cnum=[1];
    cden=[-1 1];
    model=scicos_model()
    model.sim=list('rt_filtfilt',4)
    outport=ch; 
    inport=ch;
      if exists('inport') then model.in=ones(inport,1)*samples, else model.in=1, end
      if exists('outport') then model.out=ones(outport,1)*samples, else model.out=1, end
    model.evtin=[]
    model.rpar=[cnum(:);cden(:)]
    model.ipar=[ch;
		samples;
		order;
		filtfilt]
    model.dstate=[];
    model.blocktype='c'
    model.dep_ut=[%t %f]
    exprs=[sci2exp(filtfilt);sci2exp(samples);sci2exp(order);num;den];
 if (filtfilt==1) then 
    gr_i=['xstringb(orig(1),orig(2),[string(samples)+'' -> ''+string(samples);''filtfilt''],sz(1),sz(2),''fill'');']
else
     gr_i=['xstringb(orig(1),orig(2),[string(samples)+'' -> ''+string(samples);''filt''],sz(1),sz(2),''fill'');']
end
    x=standard_define([3 2],model,exprs,gr_i)
  end
endfunction
