function [status,msg]=copynewerfile(varargin)
// copy file
//
// Calling Sequence
// copynewerfile('source','destination')
// [status,message] = copynewerfile('source','destination')
//
// Description
// 
//copynewerfile('source','destination') copies the file to the file or directory destination, if the source file is newer then the destination file.
//[status,message] = copynewerfile('source','destination') copies source to destination, returning the status and a message.
//
//

lhs=argn(1);
rhs=argn(2);

fname = 'copynewerfile';

Writable=%F;
status=0;
msg='';

select rhs
case 2
  SourceFile=varargin(1);
  DestinationFile=varargin(2);
  break
case 3
  SourceFile=varargin(1);
  DestinationFile=varargin(2);
  flag=convstr(varargin(3),'u');
  if (flag=='WRITABLE') then
	Writable=%T;
  else
	error(msprintf(gettext("%s: Wrong value for input argument #%d: %s expected.\n"),fname,3,"WRITABLE"));
  end
  break
else
  error(msprintf(gettext("%s: Wrong number of input arguments.\n"),fname));
end

[DestDir,DestFile]=GetDirFile(DestinationFile);

if (getos()=="Windows") then
  [SourceDir,SourceFile]=GetDirFile(SourceFile);
  
  Src = """"+fullfile(SourceDir,SourceFile)+"""";
  Dest = """"+fullfile(DestDir,DestFile)+"""";
  
  Rep=isdir(SourceDir);
  if (Rep==%F) then
	// Check the source directory is a directory
	msg=msprintf(gettext("%s: Source directory %s does not exist or is unreadable.\n"), fname ,SourceDir);
	return;
  end
  Rep=ls(fullfile(SourceDir,SourceFile))

else
  Src=SourceFile
  Dest=DestinationFile
  Rep=ls(Src)
end

if (Rep==[]) then
  // Check the source file existes
  msg=msprintf(gettext("%s: Source file %s does not exist or unable to access.\n"),fname,SourceFile);
  return;
end
// Now checking destination directory...
Rep=isdir(DestDir);
if (Rep==%F) then
  msg=msprintf(gettext("%s: Destination directory %s does not exist or is unreadable.\n"),fname,DestDir);
  return;
end

if (~isdir(Dest)) then
  if (fileinfo(Dest)~=[]) then			    
      if (newest(Src,Dest)==2) then
	return;
      end;
  end
else
  [SourceDir,SrcFile]=GetDirFile(SourceFile);
  if (fileinfo(fullfile(DestinationFile,SrcFile))~=[]) then			    
      if (newest(Src,fullfile(DestinationFile,SrcFile))==2) then
	return;
      end;
  end
end


if (getos()=="Windows") then
  cmd='copy '+Src+' '+Dest;
  batchlog = ' >'+ TMPDIR+'\copyfile.out' +' 2>'+TMPDIR+'\copyfile.err';
else
  // strsubst are here to "protect" the space in the path
  cmd='cp '+strsubst(Src," ","\ ")+' '+strsubst(Dest," ","\ ");
  batchlog = ' >'+ TMPDIR+'/copyfile.out' +' 2>'+TMPDIR+'/copyfile.err';
end
cmdline =cmd+batchlog;
// TODO: Rewrite this stuff
status=unix(cmdline);

if (status~=0) then
  if (getos()=="Windows") then
	msg='Error : '+mgetl(TMPDIR+'\copyfile.err');
	msg=msg+' '+mgetl(TMPDIR+'\copyfile.out');
  else
	msg='Error : '+mgetl(TMPDIR+'/copyfile.err');
	msg=msg+' '+mgetl(TMPDIR+'/copyfile.out');
  end
  status=0;
else
  msg='';
  status=1;
end

if (Writable==%T) & (status==1) then
  if (getos()=="Windows") then
	cmd='attrib -r '+ Dest;
	batchlog = ' >'+ TMPDIR+'\attrib.out' +' 2>'+TMPDIR+'\attrib.err';
  else
	cmd='chmod +w '+ Dest;
	batchlog = ' >'+ TMPDIR+'\attrib.out' +' 2>'+TMPDIR+'\attrib.err';
  end
  cmdline =cmd+batchlog;
  status=unix(cmdline);
  if (status~=0) then
	if (getos()=="Windows") then
	  msg='Error : '+mgetl(TMPDIR+'\attrib.err');
	  msg=msg+' '+mgetl(TMPDIR+'\attrib.out');
	else
	  msg='Error : '+mgetl(TMPDIR+'/attrib.err');
	  msg=msg+' '+mgetl(TMPDIR+'/attrib.out');
	end
  end
end

endfunction
//------------------------------------------------------------------------
function [Directory,Filename]=GetDirFile(OrigFile)
[path,fname,ext]=fileparts(OrigFile);
if (path=='') then
  Directory=pwd();
  Filename=OrigFile;
else
  Directory=path;
  Filename=fname+ext;
end
endfunction
//------------------------------------------------------------------------
