function [BLOCK]=hart_edit_scicosblock(target_file)
// Build a interface function file . 
//
// Calling Sequence
//  BLOCK = hart_edit_scicosblock(target_file])
//
// Parameters
//  target_file: the name of .sci source file to be processed.
//  BLOCK: returns the BLOCK variable for testing
//
// Authors
//  Holger Nahrstaedt

  if argn(2)<1 | isempty(target_file) then
      target_file=pwd()+'/test.sci';
  else
      if (fileparts(target_file)==[]) then
	target_file=pwd()+target_file;
      end
  end
 target_dir=pwd()
//tkpath = get_absolute_file_path("gen_block.sci");
tkpath = fileparts(get_function_path('hart_edit_scicosblock'));
ending='*.sci';


try
  getversion('scilab');
  scilabver=5;
catch
  scilabver=4;
end
if(scilabver==5) then
  pg=list();
  pg($+1)=list(list('pushbutton','Target file','callback','set(findobj(''''tag'''',''''TDir''''),''''string'''',uigetfile(""""'+ending+'"""",""""'+target_dir+'""""))'),list(2,'edit',target_file,'tag','TDir'));
  pg($+1)=list(list('pushbutton','OK','Horizontalalignment','center','callback','OK=%T'),list('pushbutton','Cancel','Horizontalalignment','center','callback','CANCEL=%T'));
  [target_file]=guimaker(pg,list("block_creator",650));
else
  target_file=tk_savefile(ending,pwd());
end


if isempty(target_file) then disp('Terminated by user...nothing changed.'); return; end

 [path,name,extension]=fileparts(target_file(1));
 
printf('Creating a new scicos block...\n');
printf(' ...in the target directory: %s\n',path);
printf(' ...for the module: %s\n',name);
currentdir=pwd();
chdir(path);
status=fileinfo(name+extension);
if status~=[] then
  if (scilabver==5) then
    answ=messagebox(name+extension+" already exists","modal","info",["Overwrite" "Cancel"]);
  else
    //answ=x_message([name+extension+" already exists"],["Overwrite" "Cancel"]);
	answ=1;
  end

  if answ==2 then return; chdir(currentdir);end
  printf(' %s already exists - overwriting file...\n',name+extension);

  f=mopen(target_file(1),'r');
  help_txt=[];
line=' '
while isempty(strindex(line,'function ')) & ~meof(f), line=mgetl(f,1); end 
line=mgetl(f,1); 
while (~isempty(stripblanks(line)) & ~meof(f))
  if ~isempty(strindex(line,'//')) then
    help_txt= [ help_txt; line;]
  end;
line=mgetl(f,1); 
end

mclose(f);

  exec(target_file(1),-1);
  execstr('BLOCK='+name+'(''readout'')');
  clear(target_file(1));


  if BLOCK.version==10 then

    BLOCK.opar=[];
    BLOCK.in2 = '';
    BLOCK.intype = 1;
    BLOCK.out2 = '';
    BLOCK.outtype = 1;
    BLOCK.IOmatrix = %f;
    BLOCK.inset=%f
    BLOCK.outset=%f
  end;







end

if (~exists('BLOCK')) then BLOCK=[]; end;

if max(size(BLOCK))==0 then
  BLOCK.name='test';
  BLOCK.comp_name='rt_par2ser';
  BLOCK.desr_short='parallel to serial block';
  BLOCK.dep_u=%f;
  BLOCK.dep_t=%f;
  BLOCK.blocktype='d';
  BLOCK.dstate='';
    BLOCK.IOmatrix = %f;
  BLOCK.inset=%t
  BLOCK.in='ones(ch,1)*samples';
  BLOCK.outset=%f
  BLOCK.out='ones(ch,1)';
  
  BLOCK.evtin='1';
  BLOCK.evtout='';
  BLOCK.size='3 2';
  BLOCK.ipar=[];
  BLOCK.rpar=[];
  BLOCK.opar=[];
  BLOCK.completelabel=%f;
  BLOCK.label=[];
  BLOCK.parameter=list();
 BLOCK.parameter($+1)=[];
BLOCK.parameter(1).name='ch';
BLOCK.parameter(1).text='Channels:';
BLOCK.parameter(1).type='vec';
BLOCK.parameter(1).size='-1';
BLOCK.parameter(1).init='1';
BLOCK.parameter(1).visible_plot=%F;
end;
if (exists(name)) then clear(name); end;
BLOCK.name=name;
clear name;
if(scilabver==5) then
TCL_UnsetVar("block");
TCL_UnsetVar("var");
TCL_UnsetVar("parameter");
TCL_UnsetVar("param");
TCL_UnsetVar("paramvar");
TCL_UnsetVar("save");
else
TCL_EvalStr("if {[info exists block]} {unset block}");
TCL_EvalStr("if {[info exists var]} {unset var}");
TCL_EvalStr("if {[info exists parameter]} {unset parameter}");
TCL_EvalStr("if {[info exists param]} {unset param}");
TCL_EvalStr("if {[info exists paramvar]} {unset paramvar}");
TCL_EvalStr("if {[info exists save]} {unset save}");
end;
TCL_SetVar("block(name)",BLOCK.name);
TCL_SetVar("block(comp_name)",BLOCK.comp_name);
TCL_SetVar("block(desr_short)",BLOCK.desr_short);
if (BLOCK.dep_u) then
  TCL_SetVar("block(dep_u)","1");
else
  TCL_SetVar("block(dep_u)","0");
end;

if (BLOCK.dep_t) then
  TCL_SetVar("block(dep_t)","1");
else
  TCL_SetVar("block(dep_t)","0");
end;
TCL_SetVar("block(blocktype)",BLOCK.blocktype);
TCL_SetVar("block(dstate)",BLOCK.dstate);
if (BLOCK.IOmatrix) then
  TCL_SetVar("block(IOmatrix)","1");
else
  TCL_SetVar("block(IOmatrix)","0");
end;
if (BLOCK.inset) then
  TCL_SetVar("block(inset)","1");
else
  TCL_SetVar("block(inset)","0");
end;
TCL_SetVar("block(in)",BLOCK.in);
if (BLOCK.IOmatrix) then
  TCL_SetVar("block(in2)",BLOCK.in2);
  TCL_SetVar("block(intype)",BLOCK.intype);
end;
// select BLOCK.intype,
//   case 1 then   TCL_SetVar("block(intype)","real"),
//   case 2 then   TCL_SetVar("block(intype)","complex"),
//   case 3 then   TCL_SetVar("block(intype)","int32"),
//   case 4 then   TCL_SetVar("block(intype)","int16"),
//   case 5 then   TCL_SetVar("block(intype)","int8"),
//   case 6 then   TCL_SetVar("block(intype)","uint32"),
//   case 7 then   TCL_SetVar("block(intype)","uint16"),
//   case 8 then   TCL_SetVar("block(intype)","uint8"),
// end
if (BLOCK.outset) then
  TCL_SetVar("block(outset)","1");
else
  TCL_SetVar("block(outset)","0");
end;
TCL_SetVar("block(out)",BLOCK.out);
if (BLOCK.IOmatrix) then
  TCL_SetVar("block(out2)",BLOCK.out2);
  TCL_SetVar("block(outtype)",BLOCK.outtype)
end;
// select BLOCK.outtype,
//   case 1 then   TCL_SetVar("block(outtype)","real"),
//   case 2 then   TCL_SetVar("block(outtype)","complex"),
//   case 3 then   TCL_SetVar("block(outtype)","int32"),
//   case 4 then   TCL_SetVar("block(outtype)","int16"),
//   case 5 then   TCL_SetVar("block(outtype)","int8"),
//   case 6 then   TCL_SetVar("block(outtype)","uint32"),
//   case 7 then   TCL_SetVar("block(outtype)","uint16"),
//   case 8 then   TCL_SetVar("block(outtype)","uint8"),
// end

TCL_SetVar("block(evtin)",BLOCK.evtin);
TCL_SetVar("block(evtout)",BLOCK.evtout);
TCL_SetVar("block(size)",BLOCK.size);
TCL_SetVar("block(ipar)",ascii([BLOCK.ipar]));
TCL_SetVar("block(rpar)",ascii([BLOCK.rpar]));
TCL_SetVar("block(opar)",ascii([BLOCK.opar]));
//TCL_SetVar("block(label)",(strsubst(BLOCK.label,'''','''''')));
if (BLOCK.completelabel) then
  TCL_SetVar("block(completelabel)","1");
else
  TCL_SetVar("block(completelabel)","0");
end;
TCL_SetVar("block(label)",ascii([BLOCK.label]));

if size(BLOCK.parameter)>0 then
   TCL_SetVar("parameter",string(size(BLOCK.parameter)));
  for (i=1:size(BLOCK.parameter))
  TCL_SetVar("param("+string(i)+")(name)",BLOCK.parameter(i).name);
  TCL_SetVar("param("+string(i)+")(text)",BLOCK.parameter(i).text);
  TCL_SetVar("param("+string(i)+")(type)",BLOCK.parameter(i).type);
  TCL_SetVar("param("+string(i)+")(size)",BLOCK.parameter(i).size);
  TCL_SetVar("param("+string(i)+")(init)",BLOCK.parameter(i).init);
  if (BLOCK.parameter(i).visible_plot) then 
    TCL_SetVar("param("+string(i)+")(visible_plot)","1");
  else
    TCL_SetVar("param("+string(i)+")(visible_plot)","0");
  end
  end;
else
    TCL_SetVar("parameter",string(0));
end;

TCL_EvalFile(tkpath+'hart_edit_scicosblock.tcl')

while %t //wait for toplevel to disapear
  parameter=eval(TCL_GetVar('parameter'));
if (scilabver==5) then
  block=TCL_GetVar('block');
  var=TCL_GetVar('var');
  if (parameter>0) then
    param=TCL_GetVar('param');
    paramvar=TCL_GetVar('paramvar');
  end;
end;
  save_sci=TCL_GetVar('save');
  TCL_EvalStr('set h [winfo exists .foo]');
  if TCL_GetVar("h")=='0' then break,end
  sleep(1);
end
if (scilabver==5) then
 block=TCL_GetVar('block');
end;
 if save_sci=='0' then disp('Terminated by user...nothing changed.'); return; end


if (scilabver==5) then
  BLOCK.name=block(find(var=='name'));
  BLOCK.comp_name=block(find(var=='comp_name'));
  BLOCK.desr_short=block(find(var=='desr_short'));
  if (block(find(var=='dep_u'))=='0') then
    BLOCK.dep_u=%F;
  else
     BLOCK.dep_u=%T;
  end
  if (block(find(var=='dep_t'))=='0') then
    BLOCK.dep_t=%F;
  else
     BLOCK.dep_t=%T;
  end
  BLOCK.blocktype=block(find(var=='blocktype'));
  BLOCK.dstate=block(find(var=='dstate'));
  if (block(find(var=='IOmatrix'))=='0') then
    BLOCK.IOmatrix=%F;
  else
     BLOCK.IOmatrix=%T;
  end
  if (block(find(var=='inset'))=='0') then
    BLOCK.inset=%F;
  else
     BLOCK.inset=%T;
  end
  BLOCK.in=block(find(var=='in'));
if (BLOCK.IOmatrix) then
  BLOCK.in2=block(find(var=='in2'));
  BLOCK.intype=block(find(var=='intype'));
end;
//   select block(find(var=='intype')),
//     case "real" then BLOCK.intype=1,
//     case "complex" then BLOCK.intype=2,
//     case "int32" then BLOCK.intype=3,
//     case "int16" then BLOCK.intype=4,
//     case "int8" then BLOCK.intype=5,
//     case "uint32" then BLOCK.intype=6,
//     case "uint16" then BLOCK.intype=7,
//     case "uint8" then BLOCK.intype=8,
//   end
  if (block(find(var=='outset'))=='0') then
    BLOCK.outset=%F;
  else
     BLOCK.outset=%T;
  end
  BLOCK.out=block(find(var=='out'));
if (BLOCK.IOmatrix) then
  BLOCK.out2=block(find(var=='out2'));
  BLOCK.outtype=block(find(var=='outtype'));
end;
//   select block(find(var=='outtype')),
//     case "real" then BLOCK.outtype=1,
//     case "complex" then BLOCK.outtype=2,
//     case "int32" then BLOCK.outtype=3,
//     case "int16" then BLOCK.outtype=4,
//     case "int8" then BLOCK.outtype=5,
//     case "uint32" then BLOCK.outtype=6,
//     case "uint16" then BLOCK.outtype=7,
//     case "uint8" then BLOCK.outtype=8,
//   end

  BLOCK.evtin=block(find(var=='evtin'));
  BLOCK.evtout=block(find(var=='evtout'));
  BLOCK.size=block(find(var=='size'));
  BLOCK.ipar=block(find(var=='ipar'));
  BLOCK.rpar=block(find(var=='rpar'));
  BLOCK.opar=block(find(var=='opar'));
  BLOCK.label=block(find(var=='label'));
  if (block(find(var=='completelabel'))=='0') then
    BLOCK.completelabel=%F;
  else
     BLOCK.completelabel=%T;
  end
  BLOCK.parameter=list();

  if (parameter>0) then
  
    for i=1:parameter 
	BLOCK.parameter($+1)=[];
	BLOCK.parameter(i).name=param(find(paramvar=='name')+((i-1)*6))
	BLOCK.parameter(i).text=param(find(paramvar=='text')+((i-1)*6))
	BLOCK.parameter(i).type=param(find(paramvar=='type')+((i-1)*6))
	BLOCK.parameter(i).size=param(find(paramvar=='size')+((i-1)*6))
	BLOCK.parameter(i).init=param(find(paramvar=='init')+((i-1)*6))
	if (param(find(paramvar=='visible_plot')+((i-1)*6))=='0') then
	  BLOCK.parameter(i).visible_plot=%f;
	else
	  BLOCK.parameter(i).visible_plot=%t;
	end

    end;
  end
else
  BLOCK.name=TCL_GetVar('block(name)');
  BLOCK.comp_name=TCL_GetVar('block(comp_name)');
  BLOCK.desr_short=TCL_GetVar('block(desr_short)');
  if (TCL_GetVar('block(dep_u)')=='0') then
    BLOCK.dep_u=%F;
  else
     BLOCK.dep_u=%T;
  end
  if (TCL_GetVar('block(dep_t)')=='0') then
    BLOCK.dep_t=%F;
  else
     BLOCK.dep_t=%T;
  end
  BLOCK.blocktype=TCL_GetVar('block(blocktype)');
  BLOCK.dstate=TCL_GetVar('block(dstate)');
  if (TCL_GetVar('block(IOmatrix)')=='0') then
    BLOCK.IOmatrix=%F;
  else
     BLOCK.IOmatrix=%T;
  end
  if (TCL_GetVar('block(inset)')=='0') then
    BLOCK.inset=%F;
  else
     BLOCK.inset=%T;
  end
  BLOCK.in=TCL_GetVar('block(in)');
if (BLOCK.IOmatrix) then
  BLOCK.in2=TCL_GetVar('block(in2)');
  BLOCK.intype=TCL_GetVar('block(intype)');
end;
//   select TCL_GetVar('block(intype)'),
//     case "real" then BLOCK.intype=1,
//     case "complex" then BLOCK.intype=2,
//     case "int32" then BLOCK.intype=3,
//     case "int16" then BLOCK.intype=4,
//     case "int8" then BLOCK.intype=5,
//     case "uint32" then BLOCK.intype=6,
//     case "uint16" then BLOCK.intype=7,
//     case "uint8" then BLOCK.intype=8,
//   end
  if (TCL_GetVar('block(outset)')=='0') then
    BLOCK.outset=%F;
  else
     BLOCK.outset=%T;
  end
  BLOCK.out=TCL_GetVar('block(out)');
if (BLOCK.IOmatrix) then
  BLOCK.out2=TCL_GetVar('block(out2)');
  BLOCK.outtype=TCL_GetVar('block(outtype)');
end;
//   select TCL_GetVar('block(outtype)'),
//     case "real" then BLOCK.outtype=1,
//     case "complex" then BLOCK.outtype=2,
//     case "int32" then BLOCK.outtype=3,
//     case "int16" then BLOCK.outtype=4,
//     case "int8" then BLOCK.outtype=5,
//     case "uint32" then BLOCK.outtype=6,
//     case "uint16" then BLOCK.outtype=7,
//     case "uint8" then BLOCK.outtype=8,
//   end

  BLOCK.evtin=TCL_GetVar('block(evtin)');
  BLOCK.evtout=TCL_GetVar('block(evtout)');
  BLOCK.size=TCL_GetVar('block(size)');
  BLOCK.ipar=TCL_GetVar('block(ipar)');
  BLOCK.rpar=TCL_GetVar('block(rpar)');
  BLOCK.opar=TCL_GetVar('block(opar)');
  BLOCK.label=TCL_GetVar('block(label)');
  if (TCL_GetVar('block(completelabel)')=='0') then
    BLOCK.completelabel=%F;
  else
     BLOCK.completelabel=%T;
  end
  BLOCK.parameter=list();

  if (parameter>0) then
  
    for i=1:parameter 
	BLOCK.parameter($+1)=[];
	BLOCK.parameter(i).name=TCL_GetVar('param('+string(i)+')(name)')
	BLOCK.parameter(i).text=TCL_GetVar('param('+string(i)+')(text)')
	BLOCK.parameter(i).type=TCL_GetVar('param('+string(i)+')(type)')
	BLOCK.parameter(i).size=TCL_GetVar('param('+string(i)+')(size)')
	BLOCK.parameter(i).init=TCL_GetVar('param('+string(i)+')(init)')
	if (TCL_GetVar('param('+string(i)+')(visible_plot)')=='0') then
	  BLOCK.parameter(i).visible_plot=%f;
	else
	  BLOCK.parameter(i).visible_plot=%t;
	end

    end;
  end

end
sci_txt=['function [x,y,typ] = '+BLOCK.name+'(job,arg1,arg2)'];
if (exists('help_txt')) then
sci_txt=[sci_txt;help_txt];
end;
sci_txt=[sci_txt;''];


//help
sci_txt=[sci_txt;
'  x=[];y=[];typ=[];'
'  select job'
'  case ''plot'' then'
'    exprs=arg1.graphics.exprs;'
];

for i=1:size(BLOCK.parameter)
    if (BLOCK.parameter(i).visible_plot) then
      sci_txt=[sci_txt;'    '+BLOCK.parameter(i).name+'=exprs('+string(i)+');'];
    end;
end;

sci_txt=[sci_txt;
'    standard_draw(arg1)'
'  case ''getinputs'' then'
'    [x,y,typ]=standard_inputs(arg1)'
'  case ''getoutputs'' then'
'    [x,y,typ]=standard_outputs(arg1)'
'  case ''getorigin'' then'
'    [x,y]=standard_origin(arg1)'
'  case ''set'' then'
'    x=arg1'
'    model=arg1.model;graphics=arg1.graphics;'
'    exprs=graphics.exprs;'
'    while %t do'
];

if (size(BLOCK.parameter)>0) then
  sci_txt=[sci_txt;
'  try'
'  getversion(''scilab'');'
];
  tmp_str=['      [ok,'];
  for i=1:size(BLOCK.parameter)
    tmp_str=tmp_str+BLOCK.parameter(i).name;
    if i<size(BLOCK.parameter) then
    tmp_str=tmp_str+',';
    end
  end;
    tmp_str=tmp_str+',exprs]=..';
    sci_txt=[sci_txt;tmp_str];
    sci_txt=[sci_txt;'      scicos_getvalue('''+BLOCK.desr_short+''',..'];
    for i=1:size(BLOCK.parameter)
      if size(BLOCK.parameter)==1 then
	   sci_txt=[sci_txt;'        ['''+BLOCK.parameter(i).text+'''],..'];
      elseif i==1 then
	  sci_txt=[sci_txt;'         ['''+BLOCK.parameter(i).text+''';'];
      elseif i==size(BLOCK.parameter)
	  sci_txt=[sci_txt;'          '''+BLOCK.parameter(i).text+'''],..'];
      else
	sci_txt=[sci_txt;'         '''+BLOCK.parameter(i).text+''';'];
      end;
    end;

  tmp_str=['      list('];
  for i=1:size(BLOCK.parameter)
  tmp_str=tmp_str+''''+BLOCK.parameter(i).type+''','+BLOCK.parameter(i).size;
  if (i<size(BLOCK.parameter)) then
    tmp_str=tmp_str+',';
  end
  end
  tmp_str=tmp_str+'),exprs)';
    sci_txt=[sci_txt;tmp_str];
  sci_txt=[sci_txt;'catch'];
  tmp_str=['      [ok,'];
  for i=1:size(BLOCK.parameter)
    tmp_str=tmp_str+BLOCK.parameter(i).name;
    if i<size(BLOCK.parameter) then
    tmp_str=tmp_str+',';
    end
  end;
    tmp_str=tmp_str+',exprs]=..';
    sci_txt=[sci_txt;tmp_str];
    sci_txt=[sci_txt;'      getvalue('''+BLOCK.desr_short+''',..'];
    for i=1:size(BLOCK.parameter)
      if size(BLOCK.parameter)==1 then
	   sci_txt=[sci_txt;'        ['''+BLOCK.parameter(i).text+'''],..'];
      elseif i==1 then
	  sci_txt=[sci_txt;'         ['''+BLOCK.parameter(i).text+''';'];
      elseif i==size(BLOCK.parameter)
	  sci_txt=[sci_txt;'          '''+BLOCK.parameter(i).text+'''],..'];
      else
	sci_txt=[sci_txt;'         '''+BLOCK.parameter(i).text+''';'];
      end;
    end;

  tmp_str=['      list('];
  for i=1:size(BLOCK.parameter)
  tmp_str=tmp_str+''''+BLOCK.parameter(i).type+''','+BLOCK.parameter(i).size;
  if (i<size(BLOCK.parameter)) then
    tmp_str=tmp_str+',';
  end
  end
  tmp_str=tmp_str+'),exprs)';
    sci_txt=[sci_txt;tmp_str];
  sci_txt=[sci_txt;'end;'];
  sci_txt=[sci_txt;'     if ~ok then break,end'];
end;
if (BLOCK.inset) then
 sci_txt=[sci_txt;'      in=['+BLOCK.in+']'];
else
 sci_txt=[sci_txt;'      in=[model.in]'];
end;
if (BLOCK.IOmatrix) then
  sci_txt=[sci_txt;'      intyp=['+(BLOCK.intype)+']'];
  if (BLOCK.inset) then
  sci_txt=[sci_txt;'      in2=['+BLOCK.in2+']'];
  else
     sci_txt=[sci_txt;'      in2=[model.in2]'];
     
  end
end;

if (BLOCK.outset) then
 sci_txt=[sci_txt;'      out=['+BLOCK.out+']'];
else
 sci_txt=[sci_txt;'      out=[model.out]'];
end
if (BLOCK.IOmatrix) then
  sci_txt=[sci_txt;'      outtyp=['+(BLOCK.outtype)+']'];
  if (BLOCK.outset) then
  sci_txt=[sci_txt;'      out2=['+BLOCK.out2+']'];
  else
    sci_txt=[sci_txt;'      out2=[model.out2]'];
  end
end;
 sci_txt=[sci_txt;'      evtin=['+BLOCK.evtin+']'];
 sci_txt=[sci_txt;'      evtout=['+BLOCK.evtout+']'];
if (BLOCK.IOmatrix) then
 sci_txt=[sci_txt;
                  '      [model,graphics,ok]=set_io(model,graphics,list([in,in2],intyp),list([out,out2],outtyp),evtin,evtout,[],[]);'
'      if ok then'
'        graphics.exprs=exprs;'
];
else
 sci_txt=[sci_txt;
                  '      [model,graphics,ok]=check_io(model,graphics,in,out,evtin,evtout);'
'      if ok then'
'        graphics.exprs=exprs;'
];
end;
sci_txt=[sci_txt;'        model.ipar=['+BLOCK.ipar+'];'];
sci_txt=[sci_txt;'        model.rpar=['+BLOCK.rpar+'];'];
sci_txt=[sci_txt;'	  model.dstate=['+BLOCK.dstate+'];'];
sci_txt=[sci_txt;
                 '        x.graphics=graphics;x.model=model'
'        break'
'      end'
'    end'
'  case ''define'' then'
];

for  i=1:size(BLOCK.parameter)
  if BLOCK.parameter(i).type=='str' then
   sci_txt=[sci_txt;'     '+BLOCK.parameter(i).name+'='''+string(BLOCK.parameter(i).init)+''';'];
  else
    if (string(BLOCK.parameter(i).init)=='') then
      sci_txt=[sci_txt;'     '+BLOCK.parameter(i).name+'=[];'];
    else
      sci_txt=[sci_txt;'     '+BLOCK.parameter(i).name+'='+string(BLOCK.parameter(i).init)+';'];
    end
  end;
end;

  sci_txt=[sci_txt;'   model=scicos_model()'];
  sci_txt=[sci_txt;'   model.sim=list('''+BLOCK.comp_name+''',4)'];
  sci_txt=[sci_txt;'   model.in=['+BLOCK.in+']'];

if (BLOCK.IOmatrix) then
  if BLOCK.in2~='' then 
      sci_txt=[sci_txt;'   model.in2=['+BLOCK.in2+']'];
  end;
   if BLOCK.intype~='' then 
    sci_txt=[sci_txt;'   model.intyp=['+(BLOCK.intype)+']'];
  end;
end;
  sci_txt=[sci_txt;'   model.out=['+BLOCK.out+']'];
if (BLOCK.IOmatrix) then
  if BLOCK.out2~='' then 
      sci_txt=[sci_txt;'   model.out2=['+BLOCK.out2+']'];
  end;
    if BLOCK.outtype~='' then
      sci_txt=[sci_txt;'   model.outtyp=['+(BLOCK.outtype)+']'];
    end;
end;
  sci_txt=[sci_txt;'   model.evtin=['+BLOCK.evtin+']'];
  sci_txt=[sci_txt;'   model.evtout=['+BLOCK.evtout+']'];
  sci_txt=[sci_txt;'   model.ipar=['+BLOCK.ipar+'];'];
  sci_txt=[sci_txt;'   model.rpar=['+BLOCK.rpar+'];'];


sci_txt=[sci_txt;'	model.dstate=['+BLOCK.dstate+'];'];
sci_txt=[sci_txt;'	model.blocktype='''+BLOCK.blocktype+''';'];
sci_txt=[sci_txt;'	model.dep_ut=['+sci2exp(BLOCK.dep_u)+' '+sci2exp(BLOCK.dep_t)+'];'];
tmp_str=['    exprs=['];

for i=1:size(BLOCK.parameter)
  if (BLOCK.parameter(i).type=='str') then
    tmp_str=tmp_str+BLOCK.parameter(i).name;
  else
     tmp_str=tmp_str+'sci2exp('+BLOCK.parameter(i).name+')';
  end
  if i<size(BLOCK.parameter) then
tmp_str=tmp_str+';';
  end
end;

tmp_str=tmp_str+']';
sci_txt=[sci_txt;tmp_str];
if (BLOCK.completelabel) then
   sci_txt=[sci_txt;'    gr_i=['+strsubst(BLOCK.label,ascii(10),ascii(32))+'];'];
else
  sci_txt=[sci_txt;'    gr_i=[''xstringb(orig(1),orig(2),['+strsubst(BLOCK.label,ascii(10),ascii(32))+'],sz(1),sz(2),''''fill'''');''];'];
end;
sci_txt=[sci_txt;'    x=standard_define(['+BLOCK.size+'],model,exprs,gr_i)'];
sci_txt=[sci_txt; 'case ''readout'' then'];
sci_txt=[sci_txt;'      BLOCK.version=020;'];
sci_txt=[sci_txt;'      BLOCK.name='''+BLOCK.name+''';'];
sci_txt=[sci_txt;'      BLOCK.comp_name='''+BLOCK.comp_name+''';'];
sci_txt=[sci_txt;'      BLOCK.desr_short='''+BLOCK.desr_short+''';'];
sci_txt=[sci_txt;'      BLOCK.dep_u='+sci2exp(BLOCK.dep_u)+';'];
sci_txt=[sci_txt;'      BLOCK.dep_t='+sci2exp(BLOCK.dep_t)+';'];
sci_txt=[sci_txt;'      BLOCK.blocktype='''+BLOCK.blocktype+''';'];
sci_txt=[sci_txt;'      BLOCK.dstate='''+BLOCK.dstate+''';'];
sci_txt=[sci_txt;'      BLOCK.IOmatrix='+sci2exp(BLOCK.IOmatrix)+';'];
sci_txt=[sci_txt;'      BLOCK.inset='+sci2exp(BLOCK.inset)+';'];
sci_txt=[sci_txt;'      BLOCK.in='''+BLOCK.in+''';'];
if (BLOCK.IOmatrix) then
  sci_txt=[sci_txt;'      BLOCK.in2='''+BLOCK.in2+''';'];
  sci_txt=[sci_txt;'      BLOCK.intype='''+(BLOCK.intype)+''';'];
end;
sci_txt=[sci_txt;'      BLOCK.outset='+sci2exp(BLOCK.outset)+';'];
sci_txt=[sci_txt;'      BLOCK.out='''+BLOCK.out+''';'];
if (BLOCK.IOmatrix) then
  sci_txt=[sci_txt;'      BLOCK.out2='''+BLOCK.out2+''';'];
  sci_txt=[sci_txt;'      BLOCK.outtype='''+(BLOCK.outtype)+''';'];
end;
sci_txt=[sci_txt;'      BLOCK.evtin='''+BLOCK.evtin+''';'];
sci_txt=[sci_txt;'      BLOCK.evtout='''+BLOCK.evtout+''';'];
sci_txt=[sci_txt;'      BLOCK.size='''+BLOCK.size+''';'];
//tmpstr=sci2exp(ascii(strsubst(BLOCK.label,'''','''''')));
sci_txt=[sci_txt;'      BLOCK.completelabel='+sci2exp(BLOCK.completelabel)+';'];
tmpstr=sci2exp(ascii(BLOCK.label));
if (max(size(tmpstr))==1) then
  sci_txt=[sci_txt;'      BLOCK.label='+tmpstr+';'];
else
   sci_txt=[sci_txt;'      BLOCK.label='+tmpstr(1)];
  for i=2:(max(size(tmpstr))-1)
    sci_txt=[sci_txt;'         '+tmpstr(i)];
  end;
    sci_txt=[sci_txt;'         '+tmpstr(max(size(tmpstr)))+';'];
end
//sci_txt=[sci_txt;'      BLOCK.ipar='+sci2exp(ascii(strsubst(BLOCK.ipar,'''','''''')))+';'];
tmpstr=sci2exp(ascii(BLOCK.ipar));
if (max(size(tmpstr))==1) then
  sci_txt=[sci_txt;'      BLOCK.ipar='+tmpstr+';'];
else
   sci_txt=[sci_txt;'      BLOCK.ipar='+tmpstr(1)];
  for i=2:(max(size(tmpstr))-1)
    sci_txt=[sci_txt;'         '+tmpstr(i)];
  end;
    sci_txt=[sci_txt;'         '+tmpstr(max(size(tmpstr)))+';'];
end
//sci_txt=[sci_txt;'      BLOCK.rpar='+sci2exp(ascii(strsubst(BLOCK.rpar,'''','''''')))+';'];
tmpstr=sci2exp(ascii(BLOCK.rpar));
if (max(size(tmpstr))==1) then
  sci_txt=[sci_txt;'      BLOCK.rpar='+tmpstr+';'];
else
   sci_txt=[sci_txt;'      BLOCK.rpar='+tmpstr(1)];
  for i=2:(max(size(tmpstr))-1)
    sci_txt=[sci_txt;'         '+tmpstr(i)];
  end;
    sci_txt=[sci_txt;'         '+tmpstr(max(size(tmpstr)))+';'];
end
//sci_txt=[sci_txt;'      BLOCK.rpar='+sci2exp(ascii(strsubst(BLOCK.rpar,'''','''''')))+';'];
tmpstr=sci2exp(ascii(BLOCK.opar));
if (max(size(tmpstr))==1) then
  sci_txt=[sci_txt;'      BLOCK.opar='+tmpstr+';'];
else
   sci_txt=[sci_txt;'      BLOCK.opar='+tmpstr(1)];
  for i=2:(max(size(tmpstr))-1)
    sci_txt=[sci_txt;'         '+tmpstr(i)];
  end;
    sci_txt=[sci_txt;'         '+tmpstr(max(size(tmpstr)))+';'];
end


sci_txt=[sci_txt;'      BLOCK.parameter=list();'];

  for i=1:size(BLOCK.parameter)
    sci_txt=[sci_txt;'      BLOCK.parameter($+1)=[];'];
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').name='''+BLOCK.parameter(i).name+''';'];
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').text='''+BLOCK.parameter(i).text+''';'];
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').type='''+BLOCK.parameter(i).type+''';'];
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').size='''+BLOCK.parameter(i).size+''';'];
    if BLOCK.parameter(i).type=='str' then
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').init='''+string(BLOCK.parameter(i).init)+''';'];
    else
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').init='''+string(BLOCK.parameter(i).init)+''';'];
    end;
    sci_txt=[sci_txt;'      BLOCK.parameter('+string(i)+').visible_plot='+sci2exp(BLOCK.parameter(i).visible_plot)+';'];
  end;

sci_txt=[sci_txt;'      x=BLOCK;'];
sci_txt=[sci_txt;
'  end'
'endfunction'
];



   f=mopen(path+BLOCK.name+'.sci','w'); mfprintf(f,'%s\n',sci_txt); mclose(f);
endfunction
