function hart_module_creator(name)
// A tool for quickly setting up a new toolbox.
// Calling Sequence
//  hart_module_creator() 
//  hart_module_creator(name)
//
// Parameters
//  name: the name of the toolbox 
//
// Description
// A user friendly frontend for the Scilab toolbox template. 
//
// Select which sub directories should be included in the toolbox, 
// and define names and target directory.
//
// After the toobox template directory has been created and the relevant 
// files have been place in the macro directory, use 
// help_from_sci for automatic generation of xml help files.
//
//
// Authors
//  T. Pettersen ; torbjorn.pettersen@broadpark.no
// Bibliography
//   Cornett, A., Lipp, S. ; SCI/contrib/toolbox_skeleton

  //    Copyright (C) 2008  Torbjorn Pettersen
  //
  // This program is free software; you can redistribute it and/or modify
  // it under the terms of the GNU General Public License as published by
  // the Free Software Foundation; either version 2 of the License, or
  // (at your option) any later version.
  // 
  // This program is distributed in the hope that it will be useful,
  // but WITHOUT ANY WARRANTY; without even the implied warranty of
  // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  // GNU General Public License for more details.
  // 
  // You should have received a copy of the GNU General Public License
  // along with this program; if not, write to the Free Software
  // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//if argn(2)<2 then target_dir=pwd(); end
target_dir=fileparts(get_function_path('hart_module_creator'))+'../../modules';
if argn(2)<1 then name='test'; end;
help_name=name+' module for Hart Toolbox';
try
  getversion('scilab');
  scilabver=5;
catch
  scilabver=4;
end

pg=list();
pg($+1)=list(list('text','Name of module'),list(2,'edit',name));
if scilabver==5 then
pg($+1)=list(list('pushbutton','Target directory','callback','set(findobj(''''tag'''',''''TDir''''),''''string'''',uigetdir(""""'+target_dir+'""""))'),list(2,'edit',target_dir,'tag','TDir'));
else
pg($+1)=list(list('text','Target directory'),list(2,'edit',target_dir));
end
pg($+1)=list(list('text','Help chapter heading'),list(2,'edit',help_name));
pg($+1)=list(list('text','Template directories'),list(2,'text','Description'))
pg($+1)=list(list('checkbox','macros','value','1'),list(2,'text','Scilab macros (.sci files)'));
pg($+1)=list(list('checkbox','src','value','1'),list(2,'text','source code (all .c and .f files)'));
pg($+1)=list(list('checkbox','help','value','1'),list(2,'text','the .xml help files'));
pg($+1)=list(list('checkbox','tests'),list(2,'text','scripts to test the module'));
pg($+1)=list(list('checkbox','demos'),list(2,'text','examples to illustrate the module'));
pg($+1)=list(list('pushbutton','OK','Horizontalalignment','center','callback','OK=%T'),list('pushbutton','Cancel','Horizontalalignment','center','callback','CANCEL=%T'));
[name,target_dir,help_name,c_macros,c_src,c_help,c_tests,c_demos]=guimaker(pg,list("module_creator",650));

if isempty(name) then disp('Terminated by user...nothing changed.'); return; end

printf('Creating a module template...\n');
printf(' ...in the target directory: %s\n',target_dir);
printf(' ...for the module: %s\n',name);

chdir(target_dir);
status=mkdir(name)
if status==2 then
  answ=buttondialog(target_dir+'\'+name+' already exists',"Overwrite|Cancel");
  if answ==2 then return; end
  printf(' %s already exists - overwriting files...\n',name);
end
chdir(name);

configure_txt=[
'mode(-1);'
'lines(0);'
''];
configure_txt=[configure_txt;'TOOLBOX_NAME = '''+name+''';'];
configure_txt=[configure_txt;'TOOLBOX_TITLE = '''+help_name+''';'];
configure_txt=[configure_txt;..
'// ===================================================================='
'buildable=''yes''; //change to no, if the module should not be build'
''];

if c_src then 

configure_txt=[configure_txt;
''
'// Add the names of the c - funktions to names. E.g.: names=[''rt_file1'',''rt_file2''];'
'names =   []; '
''
'// Add the c file names which should be compiled to files. Replace the ending with .o. E.g.: files = [''file1.o'';''file2.o''];'
'files     = [];'
''
'// Add external shared librarys to ldflags'
'// E.g.: ldflags = '" -lspezial_lib'"; '
''
]; 
end;

f=mopen('configure.sce','w'); mfprintf(f,'%s\n',configure_txt); mclose(f);


txt=[
'License for the toolbox '+name
''
'Please update this file with the license terms (Copy and paste the license agreement e.g).'
''
'    Copyright (C) YEAR  Authors name'
''
' This program is free software; you can redistribute it and/or modify'
' it under the terms of the GNU General Public License as published by'
' the Free Software Foundation; either version 2 of the License, or'
' (at your option) any later version.'
' '
' This program is distributed in the hope that it will be useful,'
' but WITHOUT ANY WARRANTY; without even the implied warranty of'
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the'
' GNU General Public License for more details.'
' '
' You should have received a copy of the GNU General Public License'
' along with this program; if not, write to the Free Software'
' Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA'
''
'Please note that Scilab is released under the terms of the CeCILL license :'
'http://www.cecill.info/index.en.html'
''
];
f=mopen('license.txt','w'); mfprintf(f,'%s\n',txt); mclose(f);

txt=[
'README file for the toolbox '+name
''
'Please update this file to provide information about your module:'
' * what it is doing'
' * the author'
' * a few word about the license'
''
'Add interface functions (.sci files) in the directory '+name+'/macros'
'by using the command (you have to load the hart toolbox before)'
'   hart_edit_scicosblock()'
''
'Document the source code using the templates provided by help_from_scicosblock.'
'See help help_from_scicosblock for more information.'
'Create .xml help files using the command (run from directory '+name+')'
'   help_from_scicosblock(''macros'',''help/en_US'')'
''
'Add computational c/c++ functions to'+name+'/src/c'
''
'Update configure.sce'
'Build the module using the command:'
'   chdir(../../) // go to the hart toolbox base directory' 
'   exec prepare_toolbox.sce'
'   exec builder.sce'
'Finally load the toolbox using the command:'
'   exec loader.sce'
];
f=mopen('readme.txt','w'); mfprintf(f,'%s\n',txt); mclose(f);

if c_macros then
  mkdir('macros');
  printf(' ..creating %s/macros for Scilab function definitions (.sci files)\n',name);
end

if c_src then
  mkdir('src');
  mkdir('src/c');
  printf(' ..creating %s/src/c for .c source code\n',name);
  mkdir('src/c_RTAI');
  printf(' ..creating %s/src/c_RTAI for .c source code\n',name);
end

if c_help then
  mkdir('help');
  mkdir('help/en_US');
  mkdir('help/en_US_scilab4');
  printf(' ..creating %s/help/en_US for english .xml help files\n',name);
end


if c_tests then
  mkdir('tests');
  printf(' ..creating %s/tests for unit test files (.sce)\n',name);
end

if c_demos then
  mkdir('demos');
  printf(' ..creating %s/demos for demo files (*.dem.sce)\n',name);
  txt=[
  '// ===================================================================='
  '// Created by toolbox_creator'
  '// See SCI/contrib/toolbox_skeleton/demos/toolbox_skeleton.dem.gateway.sce'
  '// for an example on how to define subdemolist manually.'
  '// ===================================================================='
  'demopath = get_absolute_file_path(""'+name+'.dem.gateway.sce"");'
  ''
  'f=dir(''*.sce''); f=f.name;' 
  'i=find(f<>name+''.dem.gateway.sce''; f=f(i);'
  'subdemolist=[''demo ''+strsubst(f,''.dem.sce'','''') f];'
  'subdemolist(:,2) = demopath + subdemolist(:,2);'
  '// ===================================================================='
  ];
  f=mopen('demos/'+name+'.dem.gateway.sce','w'); mfprintf(f,'%s\n',txt); mclose(f);  
end
chdir('..\');

if c_help then
  printf('Run help_from_scicosblock() to create a scilab function template with\n');
  printf('documentation in the .sci source files to be saved in %s/macros.\n',name);
  printf('See help help_from_scicosblock for more information.\n');
  printf('Run help_from_scicosblock(''%s/macros'',''help/en_US'') to generate .xml help files\n',name);
  printf('from the .sci source code.\n');
end
printf('Run exec prepare_toolbox.sce from the directory %s, to build the toolbox.\n',fileparts(get_function_path('hart_module_creator'))+'../../');
printf('Run exec builder.sce from the directory %s, to build the toolbox.\n',fileparts(get_function_path('hart_module_creator'))+'../../');
printf('Run exec loader.sce from the directory %s, to load the toolbox.\n',fileparts(get_function_path('hart_module_creator'))+'../../');
endfunction

