/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */

#include <machine.h>
#include <scicos_block4.h>
#include "iir_filter.h"

//Array Laenge
#define ARRAY_LENGTH 10000

struct FiltFiltDev{
	double *dataout1;
	double *dataout2;
	double *data1;
	double *data2;
	int filtfilt;
	unsigned int datacount;
	unsigned int dataindex;
	unsigned int N;
	unsigned int channel;
	unsigned int samples;
	  miirfilter Q;
	unsigned int order;
};

static int init(scicos_block *block)
{
  struct FiltFiltDev * comdev = (struct FiltFiltDev *) malloc(sizeof(struct FiltFiltDev));
	  int i;
  comdev->channel=block->ipar[0];
  comdev->samples=block->ipar[1];
  comdev->order=block->ipar[2];
  comdev->filtfilt=block->ipar[3];
  comdev->datacount=0;
  comdev->N=comdev->samples;

  comdev->dataout1=(double *) calloc((comdev->samples),sizeof(double));
  comdev->data1=(double *) calloc((comdev->samples),sizeof(double));


//   comdev->Q.ntaps=comdev->order+1;
// 	for (i=0; i<comdev->Q.ntaps; i++) {
// 		comdev->Q.a[comdev->order-i]=block->rpar[i];
// 		comdev->Q.b[comdev->order-i]=block->rpar[comdev->Q.ntaps+i];
// 	}
// if (comdev->order>0) {
// 		mIIRfilter(&comdev->Q,comdev->data1,comdev->dataout1,comdev->samples);
// 			//mIIRfilter(&comdev->Q,comdev->stellData,comdev->Qtemp,comdev->Nswing-comdev->deltas);
// 			//for (i=0;i<comdev->Nswing-comdev->deltas;i++)
// 			//{ comdev->stellData[i]=comdev->Qtemp[i];}
// 	}
  
  comdev->Q.ntaps=comdev->order+1;
	for (i=0; i<comdev->Q.ntaps; i++) {
		comdev->Q.a[comdev->order-i]=block->rpar[i];
		comdev->Q.b[comdev->order-i]=block->rpar[comdev->Q.ntaps+i];
	}
  comdev->Q.gain=1.;
if (comdev->order>0) {
		mIIRfilter(&comdev->Q,comdev->data1,comdev->dataout1,comdev->samples);
			//mIIRfilter(&comdev->Q,comdev->stellData,comdev->Qtemp,comdev->Nswing-comdev->deltas);
			//for (i=0;i<comdev->Nswing-comdev->deltas;i++)
			//{ comdev->stellData[i]=comdev->Qtemp[i];}
	}

  comdev->dataindex=0;
//bilinear_real_pole(1,100,&comdev->Q);
//dumpIIRfilt(comdev->Q);
  *block->work=(void *)comdev;
  return 0;
}
 





static int inout(scicos_block *block)
{
  int i;
double *u1,*u2;
  double *y1,*y2;
  struct FiltFiltDev * comdev = (struct FiltFiltDev *) (*block->work);

  u1 =GetRealInPortPtrs(block,1);
  y1 =GetRealOutPortPtrs(block,1);

  if (  comdev->channel==2){
	  u2 =GetRealInPortPtrs(block,2);
	  y2 =GetRealOutPortPtrs(block,2);
  }

//    if(comdev->N<(comdev->samples)){
// 	comdev->N++;
//    } else {
	for (i=0;i<comdev->samples;i++){
	  comdev->data1[i]=u1[i];
	  if (  comdev->channel==2)
    		comdev->data2[i]=u2[i];
	  
  	}
	comdev->data1[0]=0.;comdev->data1[comdev->samples-1]=0.;
	if (comdev->order>0) {
		if (comdev->filtfilt==1)
			mIIRfiltfilt(&comdev->Q,u1,y1,comdev->samples);
		else
			mIIRfilter(&comdev->Q,u1,y1,comdev->samples);
		//mIIRfilter(&comdev->Q,comdev->stellData,comdev->Qtemp,comdev->Nswing-comdev->deltas);
		//mIIRfilter(&comdev->Q,comdev->data1,comdev->dataout1,comdev->samples);
	}
// 	for (i=0;i<comdev->samples;i++){
// 	  y1[i]=comdev->dataout1[i];
// 	  if (  comdev->channel==2)
//     		y2[i]=comdev->dataout2[i];
// 	  
//   	}
// 	comdev->N=0;
// }
  
  
  return 0;
}

static int end(scicos_block *block)
{
  struct FiltFiltDev * comdev = (struct FiltFiltDev *) (*block->work);
  
  free(comdev);
  return 0;
}

void rt_filtfilt(scicos_block *block,int flag)
{

  if (flag==1){          /* get input */
    inout(block);
  }
  else if (flag==5){     /* termination */ 
    end(block);
  }
  else if (flag==4){     /* initialisation */
    init(block);
  }
}

