/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */
#include <machine.h>
#include <scicos_block4.h>


struct FlagTestDev{
	unsigned int dataindex;
	double data;
	double time;
	int step;
};

static int init(scicos_block *block)
{
  struct FlagTestDev * comdev = (struct FlagTestDev *) malloc(sizeof(struct FlagTestDev));

  printf("Flag 4: initalisation, t=%f\n",get_scicos_time());
  printf("Input ports: %d\n",GetNin(block));
  printf("Input 1: Dim: %dx%d\t Type %d\n",GetInPortRows(block,1),GetInPortCols(block,1),GetInType(block,1));

  printf("Output ports: %d\n",GetNout(block));
 printf("Output 1:  Dim: %dx%d, Type %d\n",GetOutPortRows(block,1),GetOutPortCols(block,1),GetOutType(block,1));

  comdev->dataindex=0;
  comdev->data=0.;
  comdev->step=0;
  comdev->time=get_scicos_time();
  *block->work=(void *)comdev;
 
  return 0;
}
 


static int state(scicos_block *block)
{
  struct FlagTestDev * comdev = (struct FlagTestDev *) (*block->work);
     if (get_scicos_time()==comdev->time){
	comdev->step++;
    } else {
	comdev->time=get_scicos_time();
	comdev->step=1;
    }
     printf("Flag 0: update state, step=%d, t=%f\n",comdev->step,comdev->time);

  return 0;
}
static int in(scicos_block *block)
{
struct FlagTestDev * comdev = (struct FlagTestDev *) (*block->work);
     printf(" Flag 2 ");
}

static int inout(scicos_block *block)
{
  struct FlagTestDev * comdev = (struct FlagTestDev *) (*block->work);
  int i;    
     if (get_scicos_time()==comdev->time){
	comdev->step++;
    } else {
	comdev->time=get_scicos_time();
	comdev->step=1;
    }

     printf("Flag 1: update output, step=%d, t=%f\n",comdev->step,comdev->time);
if (GetNin(block)>0){
 double *u1 = GetRealInPortPtrs(block,1);
 comdev->data=u1[0];
}
if (GetNout(block)>0){
  double *y1 = GetRealOutPortPtrs(block,1);
  y1[0]=comdev->data;
}

  return 0;
}
static int eventout(scicos_block *block)
{
  struct FlagTestDev * comdev = (struct FlagTestDev *) (*block->work);
     if (get_scicos_time()==comdev->time){
	comdev->step++;
    } else {
	comdev->time=get_scicos_time();
	comdev->step=1;
    }
     printf("Flag 3: update event output, step=%d, t=%f\n",comdev->step,comdev->time);

  return 0;
}

static int end(scicos_block *block)
{
  struct FlagTestDev * comdev = (struct FlagTestDev *) (*block->work);
     if (get_scicos_time()==comdev->time){
	comdev->step++;
    } else {
	comdev->time=get_scicos_time();
	comdev->step=1;
    }
  printf("Flag 4: ending, step=%d, t=%f\n",comdev->step,comdev->time);
  free(comdev);
  return 0;
}

void rt_flagtest(scicos_block *block,int flag)
{

  if (flag==0){    
    state(block);     /* update stats */
  }
  else if (flag==1){    
    inout(block);     /* update output */
  }
  else if (flag==2){    
    in(block);     /* ??? */
  }
  else if (flag==3){    
    eventout(block);     /* update event output */
  }
  else if (flag==5){     /* termination */ 
    end(block);
  }
  else if (flag==4){     /* initialisation */
    init(block);
  }
}

