/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */
#include <machine.h>
#include <scicos_block4.h>

struct S2PDev{
	unsigned int channels;
	unsigned int nsamples;
	unsigned int N;
	double **data;
};


static int init(scicos_block *block)
{
	struct S2PDev * comdev = (struct S2PDev *) malloc(sizeof(struct S2PDev));
	int i,j;
	int* ipar = GetIparPtrs(block);
	int nipar = GetNipar(block);
	if (nipar==2) {
		comdev->channels = ipar[0];
		comdev->nsamples = ipar[1];
	} else  {
		comdev->channels=1;
		comdev->nsamples=50;
		printf("ser2par: ipar error\n");
	}
	comdev->N = 0;

	comdev->data = (double**)malloc(comdev->channels*sizeof(double*));
	for (j = 0; j < comdev->channels; j++) {
		comdev->data[j] = (double*)malloc(comdev->nsamples*sizeof(double));
		for (i=0; i < comdev->nsamples; i++) {
			comdev->data[j][i] = 0;
		}
	}

	*block->work = (void*)comdev;

	return 0;
}


static int out(scicos_block *block)
{
	struct S2PDev * comdev = (struct S2PDev *) (*block->work);
	int i,j;
	double *y, *u;

	for(j = 0; j < comdev->channels; j++)
	{
		u = GetRealInPortPtrs(block, j+1);
		comdev->data[j][comdev->N] = *u;
	}

	if (GetNevIn(block)) { //certify that clock port was activated
		comdev->N++;
	}

	if (comdev->N == comdev->nsamples)
	{
		//printf("ser2par - output:\n");
		for(j = 0; j < comdev->channels; j++)
		{
			y = GetRealOutPortPtrs(block, j+1);
			for (i=0; i < comdev->nsamples; i++){
				y[i] = comdev->data[j][i];
				//printf("%f\n",y[i]);
			}
		}
		comdev->N = 0;
	} 

	return 0;
}


static int end(scicos_block *block)
{
	struct S2PDev * comdev = (struct S2PDev *) (*block->work);
	int j;

	for(j = 0; j < comdev->channels; j++) {
		free(comdev->data[j]);
	}
	free(comdev->data);
	free(comdev);

	return 0;
}


void rt_ser2par(scicos_block *block,int flag)
{
	if (flag==1) {      /* update output */
		out(block);
	}
	else if (flag==5) { /* termination */ 
		end(block);
	}
	else if (flag==4) { /* initialisation */
		init(block);
	}
}

