/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */

#include <machine.h>
#include <scicos_block4.h>

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/select.h>
#include <sys/termios.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>




#define STD_PORT 2602


struct SOCKETDev
{
	unsigned int samples;
	
	//The Port for listening on
	int port;	

	//Socket Filedescriptor
	int fd;
};

static int init(scicos_block *block)
{
	struct SOCKETDev * comdev = (struct SOCKETDev *) malloc(sizeof(struct SOCKETDev));

	comdev->samples=block->ipar[0];
	  
	*block->work=(void *)comdev;
	
	/** 
		Plug the 'Web Enabled Interface Connector for HART'
	*/

        //The address of the server
        struct sockaddr_in server_address;                                             

	//The listening port
        comdev->port = STD_PORT;
        
        //make sock
        comdev->fd=socket(PF_INET, SOCK_STREAM, 0);
                
        int yes=1;
        if (setsockopt(comdev->fd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) 
        {
        	perror("setsockopt");
        	exit(1);
        }
        
        server_address.sin_family = AF_INET;			//host byte order
        server_address.sin_port = htons(comdev->port);		//short, network byte order
        server_address.sin_addr.s_addr = inet_addr("127.0.0.1");
	memset(&(server_address.sin_zero), '\0', 8);		//Zero
        
	if (connect(comdev->fd,(struct sockaddr*)&server_address,sizeof(server_address)) < 0)
  	{
		return 1;
	}

	printf("Connected\n");	
	
	return 0;
}
 
static int inout(scicos_block *block)
{
	int i,j,k; //Die üblichen Verdächtigen ;)	
	double *u1,*y1;

	struct SOCKETDev * comdev = (struct SOCKETDev *) (*block->work);

	//Timing issues
        struct timeval timeout;
        timeout.tv_sec=1;
        timeout.tv_usec=0;

	//Socket stuff
        fd_set rfds, wfds, efds;
	


	u1 = GetRealInPortPtrs(block,1);
  	y1 = GetRealOutPortPtrs(block,1);

	double dd;
	
	for(j=0;j<comdev->samples;j++)
	{
				
		
		dd = (u1[j]);
		for(i=0;i<sizeof(double);)
		{
			FD_ZERO(&wfds);
        		FD_SET(comdev->fd, &wfds);
			if(select(comdev->fd+1, 0, &wfds, 0, &timeout)==-1)
			{
			 	perror("select");
				return(1);
			}
			else if(FD_ISSET(comdev->fd, &wfds))
			{
				//send data to socket		
				k = send(comdev->fd, (char*)(&dd+i), sizeof(double), 0);	
		       		if(k != sizeof(double))	
		      		{
		     	        	perror("send");
					return(1);
		                }
				else
				{
					i+=k;
				}	
			}
		}


		for(i=0;i<sizeof(double);)
		{
			FD_ZERO(&rfds);
        		FD_SET(comdev->fd, &rfds);
			if(select(comdev->fd+1, &rfds, 0, 0, &timeout)==-1)
			{
			 	perror("select");
				return(1);
			}
			else if(FD_ISSET(comdev->fd, &rfds))
			{
				
				k=recv(comdev->fd, (char*)((&(dd))+i), sizeof(double), 0);
				if(k==0)
				{
					close(comdev->fd);
					comdev->fd = 0;
					return(1);
				}
				else
				{
					i+=k;
				}
			}
		}
		(y1[j]) = dd;
	}
	return 0;
}

static int end(scicos_block *block)
{
  	struct SOCKETDev * comdev = (struct SOCKETDev *) (*block->work);

	//Close Socket
	close(comdev->fd);
	
	free(comdev);

  	return 0;
}

void rt_socket(scicos_block *block,int flag)
{

	//printf("FLAG:%d\n",flag);

  if (flag==1){          /* get input */
    inout(block);
  }
  else if (flag==5){     /* termination */ 
    end(block);
  }
  else if (flag==4){     /* initialisation */
    init(block);
  }
}



