#/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2007  Thomas Schauer
 *  Copyright (C) 2008  Holger Nahrstaedt
 *  Copyright (C) 2012  Ralph Stephan
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */

//########################################################################################
// MacOSX Operating System
//########################################################################################

#if (defined(__MACH__) && defined(__APPLE__) && !defined(__linux__))

#include <machine.h>
#include <scicos_block4.h>

#include <math.h>

#define USE_RTC 0
#define USE_SELECT 1
#define USE_NANOSLEEP 2
#define USE_HPET 3

#include <signal.h>
#include <unistd.h>
#include <stdio.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>

#include <sys/time.h>

static int RTSYNC_InUse[1] = {0};
static struct timeval tv = {1, 0};  /* 5 second timeout on select */

struct RTSYNC
{
    unsigned long int time_last, time_s, time_old, time_new, steps;
    int fd;
    int fq;
    fd_set readfds;
};

unsigned long int mu_time()
{
    struct timeval mytime;
    struct timezone myzone;
    gettimeofday(&mytime, &myzone);
    return (unsigned long int)(1000000 * mytime.tv_sec + mytime.tv_usec);
}

int sign(int a)
{
    if(a >= 0)
        return 1;
    else
        return -1;
}

unsigned long int timeval2li(struct timeval t)
{
    unsigned long int result;
    result = (unsigned long int)(t.tv_sec * 1000000 + t.tv_usec);
    return result;
}/* timeval2li */

struct timeval li2timeval(unsigned long int t)
{
    struct timeval result;
    result.tv_sec = t / 1000000;
    result.tv_usec = t % 1000000;
    return result;
} /* li2timeval */

struct timespec li2timespec(unsigned long int t)
{
    //ms in timespec
    struct timespec result;
    result.tv_sec = t / 1000000;
    result.tv_nsec = (t % 1000000) * 1000;
    //printf("s %d ns %d",result.tv_sec,result.tv_nsec);
    return result;
} /* li2timeval */

int init_soft_rt(scicos_block *block)
{
    struct RTSYNC * comdev = (struct RTSYNC *) malloc(sizeof(struct RTSYNC));
    int retval, ret;

    comdev->time_last = mu_time();
    comdev->time_s = comdev->time_last;
    comdev->steps = 0;
    comdev->time_old = 0;

    if(block->ipar[1] == USE_RTC)
        printf("Use RTC\n");
    else if(block->ipar[1] == USE_SELECT)
        printf("Use Select\n");
    else if(block->ipar[1] == USE_NANOSLEEP)
        printf("Use NanoSleep\n");
    else if(block->ipar[1] == USE_HPET)
        printf("Use HPET\n");
    else if(block->ipar[1] == -1)
        printf("No Waiting\n");

    *block->work = (void *)comdev;

    return 0;
}

void inout_soft_rt(scicos_block *block)
{

    struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);
    //outputs
    //time since start
    double *y1 = GetRealOutPortPtrs(block, 1);
    //time diff
    double *y2 = GetRealOutPortPtrs(block, 2);
    //spared time
    double *y3 = GetRealOutPortPtrs(block, 3);

    //printf("hart_soft_realtime inout_time = %lu\n", mu_time() );

    unsigned long int time_goal;
    int retval, ret;
    double SpareTime;
    struct timeval mytime, delay;
    struct timespec ndelay;
    unsigned long int data;
    long data2;
    unsigned char Error_Using_Clock = 0;

    comdev->steps++;

    time_goal = comdev->time_last + (block->ipar[3]) * 1000;

    if(abs(time_goal - mu_time()) > pow(2, 30))
    {
        // Wozu ist das ganze Prozedere gut ??
        if(sign(mu_time()) == -1)
            perror("Zeitueberlauf ");
        else
        {
            SpareTime = 0; //???
        }
    }
    else
    {
        if(mu_time() > time_goal)
        {
            // Hier ist zu viel Zeit vergangen
            // Rechnung hat laenger gedauert als erlaubt
            SpareTime = 0;
            printf("Failed at %f2 by %lu us\n", 0.000001 * ((double)(mu_time() - comdev->time_s)), (mu_time() - time_goal));
        }
        else
        {
            // Hier hat die Rechnung weniger lang gedauert als erlaubt
            SpareTime = (double) 0.000001 * ((double)(time_goal - mu_time()));

            // Die Zeitdifferenz wird nun durch die gewaehlte Methode aufgefuellt.
            // Es wird eine Pause fuer die Restdauer eingefuegt.
            switch(block->ipar[1])
            {
                case USE_SELECT:
                    delay = li2timeval(time_goal - mu_time());
                    select(0, 0, 0, 0, &delay);
                    break;

                case USE_RTC:
                case USE_HPET:
                case USE_NANOSLEEP:
                    ndelay = li2timespec(time_goal - mu_time() - 1000);
                    nanosleep(&ndelay, NULL);
                    break;

                default:
                    break;
            }
        }
    }

    if(block->ipar[2] == 0 && (block->ipar[1] == USE_RTC || block->ipar[1] == USE_HPET))   //only for rtc
        comdev->time_last = mu_time();
    else
        comdev->time_last = time_goal;

    comdev->time_new = mu_time() - comdev->time_s;
    y1[0] = 0.000001 * ((double)(comdev->time_new));
    gettimeofday(&mytime, NULL);
    // y2[0] = (double) (mytime.tv_sec + 1e-6*mytime.tv_usec);
    y2[0] = 0.000001 * ((double)(comdev->time_new - comdev->time_old));
    //y2[0] = y1[0]/steps;
    y3[0] = (double) SpareTime;

    comdev->time_old = comdev->time_new;
}

void end_soft_rt(scicos_block *block)
{
    struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);

    free(comdev);
}

#endif

//########################################################################################
// Linux Operating System
//########################################################################################

#if (!defined(__MACH__) && !defined(__APPLE__) && defined(__linux__))

#include <machine.h>
#include <scicos_block4.h>

#include <math.h>

#define USE_RTC 0
#define USE_SELECT 1
#define USE_NANOSLEEP 2
#define USE_HPET 3

#if defined(RTAI)

#include <rtai_lxrt.h>
#include <stdio.h>

#else

#include <signal.h>
#include <unistd.h>
#include <stdio.h>
//#include <linux/hrtimer.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>
#include <sys/time.h>
#include <sched.h>
#include <sys/poll.h>
#include <linux/rtc.h>
#include <linux/hpet.h>

#endif

#if defined(RTAI)
#else
static int RTSYNC_InUse[1] = {0};
static struct timeval tv = {1, 0};  /* 5 second timeout on select */
#endif

struct RTSYNC
{
#if defined(RTAI)
	RTIME t0, told;
	unsigned long int counter;
#else
    unsigned long int time_last, time_s, time_old, time_new, steps;
	int fd;
	int fq;
	fd_set readfds;

	int orig_scheduling_policy;
	struct sched_param orig_scheduler;

#endif
};

#if defined(RTAI)
#else
unsigned long int mu_time()
{
    struct timeval mytime;
    struct timezone myzone;
    gettimeofday(&mytime, &myzone);
    return (unsigned long int)(1000000 * mytime.tv_sec + mytime.tv_usec);
}

int sign(int a)
{
    if(a >= 0)
        return 1;
    else
        return -1;
}

unsigned long int timeval2li(struct timeval t)
{
    unsigned long int result;
    result = (unsigned long int)(t.tv_sec * 1000000 + t.tv_usec);
    return result;
}/* timeval2li */

struct timeval li2timeval(unsigned long int t)
{
    struct timeval result;
    result.tv_sec = t / 1000000;
    result.tv_usec = t % 1000000;
    return result;
} /* li2timeval */


struct timespec li2timespec(unsigned long int t)
{
    //ms in timespec
    struct timespec result;
    result.tv_sec = t / 1000000;
    result.tv_nsec = (t % 1000000) * 1000;
    //printf("s %d ns %d",result.tv_sec,result.tv_nsec);
    return result;
} /* li2timeval */

#endif

int init_soft_rt(scicos_block *block)
{
	struct RTSYNC * comdev = (struct RTSYNC *) malloc(sizeof(struct RTSYNC));
	int retval, ret;
#if defined(RTAI)
	comdev->t0 = rt_get_cpu_time_ns();
	comdev->counter = 0;
#else

	struct sched_param param, mysched;

    if(block->ipar[0] == 1)
	{
		comdev->orig_scheduling_policy = sched_getscheduler(0);
		sched_getparam(0, &comdev->orig_scheduler);
		param.sched_priority = sched_get_priority_max(SCHED_RR);
		retval = sched_setscheduler(0, SCHED_RR, &param);

        if(retval == 0)
		{
            printf("Got Real-Time Priority\n");
            mlockall(MCL_FUTURE);
		}
		else
			printf("Could no set sched priority to %d\n Please start scilab with root rights!\n", param.sched_priority);
    }

	comdev->time_last = mu_time();
	comdev->time_s = comdev->time_last;
	comdev->steps = 0;
	comdev->time_old = 0;

    if(block->ipar[1] == USE_RTC)
		printf("Use RTC\n");
    else if(block->ipar[1] == USE_SELECT)
		printf("Use Select\n");
    else if(block->ipar[1] == USE_NANOSLEEP)
        printf("Use NanoSleep\n");
    else if(block->ipar[1] == USE_HPET)
        printf("Use HPET\n");
    else if(block->ipar[1] == -1)
        printf("No Waiting\n");
#endif
	*block->work = (void *)comdev;

	return 0;
}

void inout_soft_rt(scicos_block *block)
{

    struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);
	//outputs
	//time since start
    double *y1 = GetRealOutPortPtrs(block, 1);
	//time diff
    double *y2 = GetRealOutPortPtrs(block, 2);
	//spared time
    double *y3 = GetRealOutPortPtrs(block, 3);

	//printf("hart_soft_realtime inout_time = %lu\n", mu_time() );

#if defined(RTAI)
	comdev->told = comdev->t0;
	comdev->t0 = rt_get_cpu_time_ns();
	comdev->counter++;
	//printf("time:%f,frequency %f\n",(comdev->t0 - comdev->told)*1.0E-6,1000./((comdev->t0 - comdev->told)*1.0E-6));
    y1[0] = ((double)(comdev->t0)) * 1.0E-9;
    y2[0] = ((double)(comdev->t0 - comdev->told)) * 1.0E-6;
	//y2[0] = y1[0]/steps;
	y3[0] = (double) 0;
#else

	struct sched_param param, mysched;
    struct pollfd           pfd;

	unsigned long int time_goal;
	int retval, ret;
	double SpareTime;
    struct timeval mytime, delay;
	struct timespec ndelay;
	unsigned long int data;
	long data2;
	unsigned char Error_Using_Clock = 0;

	comdev->steps++;

	time_goal = comdev->time_last + (block->ipar[3]) * 1000;

    if(abs(time_goal - mu_time()) > pow(2, 30))
	{
		// Wozu ist das ganze Prozedere gut ??
        if(sign(mu_time()) == -1)
			perror("Zeitueberlauf ");
		else
		{
			SpareTime = 0; //???
		}
	}
    else
	{
        if(mu_time() > time_goal)
		{
			// Hier ist zu viel Zeit vergangen
			// Rechnung hat laenger gedauert als erlaubt
			SpareTime = 0;
            printf("Failed at %f2 by %lu us\n", 0.000001 * ((double)(mu_time() - comdev->time_s)), (mu_time() - time_goal));
		}
		else
        {
			// Hier hat die Rechnung weniger lang gedauert als erlaubt
            SpareTime = (double) 0.000001 * ((double)(time_goal - mu_time()));

			// Die Zeitdifferenz wird nun durch die gewaehlte Methode aufgefuellt.
			// Es wird eine Pause fuer die Restdauer eingefuegt.
            switch(block->ipar[1])
			{

				case USE_RTC:
					comdev->fd = open("/dev/rtc", O_RDONLY);

                    if(comdev->fd == -1)
					{
						perror("/dev/rtc konnte nicht geoeffnet werden ");
						Error_Using_Clock = 1;
					}
					else
					{
						fflush(stderr);
						retval = ioctl(comdev->fd, RTC_PIE_ON, 0);

                        if(retval == -1)
						{
							perror("ioctl RTC_PIE_ON error ");
							Error_Using_Clock = 1;
						}
						else
						{
							comdev->fq = 1024 * 8;
							retval = ioctl(comdev->fd, RTC_IRQP_SET, comdev->fq);

                            if(retval == -1)
							{
								perror("ioctl RTC_IRQP_SET error ");
								Error_Using_Clock = 1;
							}
							else
							{

								FD_ZERO(&comdev->readfds);
								FD_SET(comdev->fd, &comdev->readfds);

                                while((mu_time() < (time_goal - 100)) && (Error_Using_Clock == 0))    //dirty hack with 100us
                                {
									retval = select(comdev->fd + 1, &comdev->readfds, NULL, NULL, &tv);

                                    if(retval == -1)
									{
										perror("Fehler bei select ");
										Error_Using_Clock = 1;
									}
									else
									{
										retval = read(comdev->fd, &data, sizeof(unsigned long));

                                        if(retval == -1)
										{
											perror("Fehler bei read ");
											Error_Using_Clock = 1;
										}
									}
								}

								retval = ioctl(comdev->fd, RTC_PIE_OFF, 0);

                                if(retval == -1)
								{
									perror("ioctl RTC_PIE_OFF error ");
									Error_Using_Clock = 1;
								}
							}
						}

						close(comdev->fd);
					}

                    if(Error_Using_Clock == 1)
					{
						perror("using nanosleep instead!\n");
						Error_Using_Clock = 0;
						printf("Please start scilab as root or do the following: sudo sysctl -w dev.rtc.max-user-freq=8192\n");
						ndelay = li2timespec(time_goal - mu_time() - 1000);
						nanosleep(&ndelay, NULL);
					}
                    break;

				case USE_SELECT:
					delay = li2timeval(time_goal - mu_time());
                    select(0, 0, 0, 0, &delay);
                    break;

				case USE_NANOSLEEP:
					ndelay = li2timespec(time_goal - mu_time() - 1000);
                    nanosleep(&ndelay, NULL);
                    break;

				case USE_HPET:
                    comdev->fd = open("/dev/hpet", O_RDONLY);

                    if(comdev->fd == -1)
					{
						perror("/dev/hpet konnte nicht geoeffnet werden ");
						Error_Using_Clock = 1;
					}
					else
					{
						fflush(stderr);
                        comdev->fq = 1024 * 8;
                        //comdev->fq = 64000;
                        retval = ioctl(comdev->fd, HPET_IRQFREQ, comdev->fq);



                        if(retval == -1)
						{
                            printf("Please start scilab as root or do the following: sudo echo 64000 > /proc/sys/dev/hpet/max-user-freq \n");
							perror("ioctl HPET_IRGFREQ error ");
							Error_Using_Clock = 1;
						}
						else
						{
                            retval = ioctl(comdev->fd, HPET_IE_ON, 0);

                            if(retval == -1)
							{
								perror("ioctl HPET_IE_ON error ");
								Error_Using_Clock = 1;
							}

							else
							{
								ioctl(comdev->fd, HPET_EPI, 0);
								//FD_ZERO(&comdev->readfds);
								//FD_SET(comdev->fd, &comdev->readfds);
								pfd.fd = comdev->fd;
								pfd.events = POLLIN;
                                while((Error_Using_Clock == 0) && (mu_time() < (time_goal - 100)))    //dirty hack with 100us
                                {
									pfd.revents = 0;
									retval =  poll(&pfd, 1, -1);
									//retval = select(comdev->fd + 1, &comdev->readfds, NULL, NULL, &tv);

                                    if(retval < 0)
									{
										perror("Fehler bei poll ");
										Error_Using_Clock = 1;
									}
									else
									{
										//retval = read(comdev->fd, &data, sizeof(unsigned long));
										retval = read(comdev->fd, &data2, sizeof(&data2));
                                        if(retval != sizeof(data))
										{
											perror("Fehler bei read ");
											Error_Using_Clock = 1;
										}
									}
								}

								retval = ioctl(comdev->fd, HPET_IE_OFF, 0);

                                if(retval == -1)
								{
									perror("ioctl HPET_IE_OFF error ");
									Error_Using_Clock = 1;
								}
							}
						}

                        close(comdev->fd);
					}

                    if(Error_Using_Clock == 1)
					{
						perror("using nanosleep instead!\n");
                        Error_Using_Clock = 0;
						printf("Please start scilab as root or do the following: sudo  echo 64000 > /proc/sys/dev/hpet/max-user-freq\n");
						ndelay = li2timespec(time_goal - mu_time() - 1000);
						nanosleep(&ndelay, NULL);
					}
                    break;

				default:
                    break;
            }
		}
	}

    if(block->ipar[2] == 0 && (block->ipar[1] == USE_RTC || block->ipar[1] == USE_HPET))   //only for rtc
		comdev->time_last = mu_time();
    else
		comdev->time_last = time_goal;

	comdev->time_new = mu_time() - comdev->time_s;
    y1[0] = 0.000001 * ((double)(comdev->time_new));
    gettimeofday(&mytime, NULL);
	// y2[0] = (double) (mytime.tv_sec + 1e-6*mytime.tv_usec);
    y2[0] = 0.000001 * ((double)(comdev->time_new - comdev->time_old));
	//y2[0] = y1[0]/steps;
	y3[0] = (double) SpareTime;

	comdev->time_old = comdev->time_new;
#endif //RTAI
}

void end_soft_rt(scicos_block *block)
{
    struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);
#if !defined(RTAI)

	struct sched_param mysched;

    if(block->ipar[0] == 1)
    {
		munlockall();
        sched_setscheduler(0, comdev->orig_scheduling_policy, &comdev->orig_scheduler);
		sched_getparam(0, &mysched);
		printf("priority reset to %i\n", mysched.sched_priority);
	}

#endif // RTAI
	free(comdev);
}

#endif

//########################################################################################
// Windows Operating System
//########################################################################################

#if (!defined(__linux__) && (defined(_WIN32) || defined(_WIN64)))


#include <scicos_block4.h>
#include <math.h>
#include <string.h>
#include <stdio.h>

#include <windows.h>
//#include <mmsystem.h>

static int RTSYNC_InUse[1] = {0};

struct RTSYNC
{
  int sampletime;  //in ms
  int ts_strict;
  int waiting;
  __int64 tprev;
  __int64  tnext;
  __int64  ts;
  __int64 tstart;
  double  PCFreq;
};

static int init_soft_rt(scicos_block *block)
{
  LARGE_INTEGER li;
  struct RTSYNC * comdev = (struct RTSYNC *) malloc(sizeof(struct RTSYNC));

  if(block->ipar[0] == 1)
  {
    if(SetPriorityClass(GetCurrentProcess(), REALTIME_PRIORITY_CLASS))
    {
      printf("Got Real-Time Priority\n");
      //mlockall(MCL_FUTURE);
    }
    else
      printf("Could no set sched priority \n ");
    }


  comdev->waiting = block->ipar[1];
  comdev->ts_strict = block->ipar[2];
  comdev->sampletime = block->ipar[3];
  //timeBeginPeriod(1); //sets the minimum timer resolution for an
  //application or device driver (in millisecond).
  //Frequenz holen

  if(!QueryPerformanceFrequency(&li))
    printf("QueryPerformanceFrequency failed!\n");

    comdev->PCFreq = (double)li.QuadPart/1000.0;

    //std::cout << "Performance Counter nicht vorhanden" << std::endl;
    comdev->ts = (__int64)(comdev->sampletime*comdev->PCFreq);

    QueryPerformanceCounter(&li);
    comdev->tprev = li.QuadPart;
    //comdev->tprev =  timeGetTime();
    comdev->tstart = comdev->tprev;
    comdev->tnext = comdev->tprev + comdev->ts;
    *block->work = (void *)comdev;
    return 0;
  }

  static int inout_soft_rt(scicos_block *block)
{
  struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);
  LARGE_INTEGER li;
  __int64 tnow;
  double SpareTime=0;
  double *y1 = GetOutPortPtrs(block, 1);
  double *y2 = GetOutPortPtrs(block, 2);
  double *y3 = GetOutPortPtrs(block, 3);
  if(comdev->waiting >-1)
  {
    do
  {
    QueryPerformanceCounter(&li);
    tnow=li.QuadPart;
    if (SpareTime == 0){
        SpareTime = (double)(tnow - comdev->tprev)/comdev->PCFreq;

    }
    Sleep(0);
    //tnow =  timeGetTime();
  }
  while(tnow < comdev->tnext);
}
else
  //tnow =  timeGetTime();
  QueryPerformanceCounter(&li);
  tnow=li.QuadPart;
  y1[0] = 1e-3*(double)(tnow - comdev->tstart)/comdev->PCFreq; //scicos time
  y2[0] =1e-3*(double)(tnow - comdev->tprev)/comdev->PCFreq; // return time elapsed since
  y3[0] = 1e-3*SpareTime;
  // previous step, convert to seconds

  comdev->tprev = tnow; // update TPREV
  if(comdev->ts_strict)
  {
    comdev->tnext += comdev->ts;  // make sure next step is TS after previous target time.
  }
  else
  {
    comdev->tnext = tnow + comdev->ts; // make sure next step is TS after current time
  }
}

static int end_soft_rt(scicos_block *block)
{
  struct RTSYNC * comdev = (struct RTSYNC *)(*block->work);

  //timeEndPeriod(1); //clears a previously set minimum timer resolution
  // see timeBeginPeriod(1)

  free(comdev);
  return 0;
}



#endif

//Parameter 1: 1 -> use RT scheduler and set to high priority (if root)
//             0 -> no RT scheduler
//Parameter 2: Realisation of the waiting
//             0 -> RTC (if max_freq_rtc=8192)
//             1 -> Select
//             2 -> Nanosleep
//Parameter 3: Compensate for failures?
//             0 -> no
//             1 -> yes -> recommended
//Parameter 4: Abtastzeit in ms
void sci_soft_rt(scicos_block *block, int flag)
{
    switch(flag)
    {
        case 1:
            inout_soft_rt(block);
            break;

        case 4:
            init_soft_rt(block);
            break;

        case 5:
            end_soft_rt(block);
            break;

        default:
            break;
    }
}
