
/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2006
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */

/*
 * Header file for libMIIR
 *
 *  $Id: miir.h,v 1.2 2006/08/31 19:25:53 hewitson Exp $
 */
#ifndef MIIR_H
#define MIIR_H

#ifndef __USE_GNU
  #define __USE_GNU
#endif
#ifndef _GNU_SOURCE
  #define _GNU_SOURCE
#endif
#define M_PIl		3.1415926535897932384626433832795029L  /* pi */

#define DEBUG 0

#define MAX_TAPS 20

#ifdef _MSC_VER
#define INLINE __forceinline /* use __forceinline (VC++ specific) */
#else
#define INLINE inline        /* use standard inline */
#endif

/*!
 * IIR filter structure to store
 * an IIR filter description.
 */
typedef struct miirfilter
{
  char    name[50];             ///< the name of the filter.
  int     fs;                   ///< the sample rate this filter is designed for.
  int     ntaps;                ///< the number of filter taps.
  double  a[MAX_TAPS];          ///< the numerator coefficients.
  double  b[MAX_TAPS];          ///< the denominator coefficients.
  double  dcgain;               ///< the gain of the filter at DC.
  double  hfgain;               ///< the gain of the filter at HF.
  double  gain;                 ///< the gain to be applied.
  double  histin[MAX_TAPS];     ///< history values.
  double  histout[MAX_TAPS];    ///< history values.

} miirfilter;


#define MAXFS 16384
/*!
 * IIR filter structure to store
 * an IIR filter description.
 */
typedef struct miirfilterAdapt
{
  char    name[50];             ///< the name of the filter
  int     fs;                   ///< the sample rate this filter is designed for
  int     ntaps;                ///< the number of filter taps
  double  a[MAX_TAPS][MAXFS];   ///< the numerator coefficients
  double  b[MAX_TAPS][MAXFS];   ///< the denominator coefficients
  double  dcgain;               ///< the gain of the filter at DC
  double  hfgain;               ///< the gain of the filter at HF
  double  gain;                 ///< the gain to be applied
  double  histin[MAX_TAPS];     ///< history values
  double  histout[MAX_TAPS];    ///< history values

} miirfilterAdapt;

/*--------------------------------------------------------------*/
/*--------------------- from tools.c ---------------------------*/

INLINE int iirnormalise(double *a, int na, double g);
INLINE double iirhfgain(double *a, double *b, int ntaps);
INLINE double iirdcgain(double *a, double *b, int ntaps);
int readIIRfilt(char *filename, miirfilter *filt);
int dumpIIRfiltToFile(miirfilter filt, char *filename);
void dumpIIRfilt(miirfilter filt);

/*--------------------------------------------------------------*/
/*--------------------- from filter.c --------------------------*/

int mIIRfilterAdapt(miirfilterAdapt *filt, double *indata, double *outdata, int ndata);
int mIIRfilter(miirfilter *filt, double *indata, double *outdata, int ndata);
int mIIRfiltfilt(miirfilter *filt, double *indata, double *outdata, int ndata);

/*--------------------------------------------------------------*/

/*--------------------- from coeffs.c --------------------------*/
/*int calc_iir_coeffs(  double **a,
                     double **b,
                      double cutoff_freq,
                      int type,
                      double ripple_percent,
                      int num_poles
                    );

*/
/*--------------------------------------------------------------*/


/*--------------------- from bilinear.c --------------------------*/
int bilinear_complex_pole(float f0, float q, int fs, miirfilter *filt);
int bilinear_complex_zero(float f0, float q, int fs, miirfilter *filt);
int bilinear_real_pole(float f0, int fs, miirfilter *filt);
int bilinear_real_zero(float f0, int fs, miirfilter *filt);
int bilinear_complex_polezero(float pf0, float pq, float zf0, float zq, int fs, miirfilter *filt);
/*--------------------------------------------------------------*/
#endif
